function [A, L, U, FreeDofs] = assemble_FV_XD(Mesh, eps, v, g, f)

    cc = computeCircumcenters(Mesh);    

    numNodes = size(Mesh.Coordinates, 1);

    % Generate stiffness matrix
    A = sparse(numNodes, numNodes);
    for i = 1:size(Mesh.Elements, 1)
        vidx = Mesh.Elements(i,:);
        A(vidx, vidx) = A(vidx, vidx) + STIMA_FV_CD(eps, v, Mesh, i, cc);
    end

    % Generate load vector
    L = zeros(numNodes, 1);
    for i = 1:size(Mesh.Elements, 1)
     vidx = Mesh.Elements(i,:);

     a12 = norm(Mesh.Coordinates(vidx(1),:)-Mesh.Coordinates(vidx(2),:)) *...
              norm(0.5*(Mesh.Coordinates(vidx(1),:)+Mesh.Coordinates(vidx(2),:)) - cc(i,:)) / 4;
     a13 = norm(Mesh.Coordinates(vidx(1),:)-Mesh.Coordinates(vidx(3),:)) *...
              norm(0.5*(Mesh.Coordinates(vidx(1),:)+Mesh.Coordinates(vidx(3),:)) - cc(i,:)) / 4;
     a23 = norm(Mesh.Coordinates(vidx(2),:)-Mesh.Coordinates(vidx(3),:)) *...
              norm(0.5*(Mesh.Coordinates(vidx(2),:)+Mesh.Coordinates(vidx(3),:)) - cc(i,:)) / 4;

      L(vidx(1)) = L(vidx(1)) + a12 + a13;
      L(vidx(2)) = L(vidx(2)) + a12 + a23;
      L(vidx(3)) = L(vidx(3)) + a13 + a23;
    end
    L = L .* f(Mesh.Coordinates);

    % Generate solution vector
    U = zeros(numNodes, 1);
    bdEdges = get_BdEdges(Mesh);
    bdVerts = Mesh.Edges(bdEdges,:);
    bdVerts = unique(bdVerts(:));
    U(bdVerts) = g(Mesh.Coordinates(bdVerts,:));

    % Generate freedofs
    FreeDofs = setdiff(1:numNodes, bdVerts);

end
